#' Odds ratio of single or collapsed variants
#'
#' @description This function simply calculates either odds ratio of each single variant or odds ratio of collapsed variants within input data.
#'
#' @param data a data frame or numeric matrix, usually a gene. Genotypes should be coded as 0,1 or 2.
#' @param y a numeric vector with two levels. Phenotype values are coded as 0 or 1.
#' @param collapse a logical value indicating whether or not to calculate odds ratio of each single variant. Default = \code{FALSE} for single variant calculation, or \code{TRUE} for collapsed variants calculation.
#' @details When \code{"collapse"} is true, a regional odds ratio will be used to represent the overall effect size and direction of variants within a genomic region. It is calculated by summing up all the allele counts to form a new contingency table, and then calcuating the regional odds ratio from the combined contingency table.
#'
#' @examples
#' data(fuzzy.zfa.example)
#' attach(fuzzy.zfa.example)
#'
#' # Single variant calculation
#' ORSingle<-odds.ratio(Z,y,collapse=FALSE)
#'
#' # Collapsed variants calculation
#' ORCollapse<-odds.ratio(Z,y,collapse=TRUE)
#'
#' @export
#' @seealso \code{\link{fuzzy.zfa}}, \code{\link{test.variants}}, \code{\link{zfa.plot}}


odds.ratio<-function(data,y,collapse=FALSE){
  if (collapse==FALSE) {
    data<-as.data.frame(data)
    pos<-colnames(data)
    result.oddsratio<-NULL
    for (i in 1:ncol(data)) {
      full.data<-cbind(data.frame(data[,i]),y)
      cont.table<-table(full.data)
      rm(full.data)
      if(sum(as.vector(cont.table)==0)>=1){
        E<-matrix(0.5,nrow=nrow(cont.table),ncol=ncol(cont.table))
        cont.table<-(E+cont.table)
      }
      if (nrow(cont.table)==2){
        result.oddsratio[i]<-((2*cont.table[1,1]+cont.table[2,1])*cont.table[2,2])/
          ((2*cont.table[1,2]+cont.table[2,2])*cont.table[2,1])
      }
      if (nrow(cont.table)==3){
        result.oddsratio[i]<-((2*cont.table[1,1]+cont.table[2,1])*(2*cont.table[3,2]+cont.table[2,2]))/
          ((2*cont.table[1,2]+cont.table[2,2])*(2*cont.table[3,1]+cont.table[2,1]))
      }
      if (nrow(cont.table)==1){
        result.oddsratio[i]<-NA
      }
      rm(cont.table)
    }
    result.oddsratio<-data.frame(pos=pos,odds.ratio=result.oddsratio)
  } else {
    y<-as.matrix(y)
    data<-as.data.frame(data)
    list.table<-apply(data,2,table,y)
    if (is.matrix(list.table)){
      cont.table<-matrix(rowSums(list.table),ncol=2)
    } else{
      all.table<-do.call(rbind,list.table)
      sum.table<-apply(all.table,2,aggregate,by=list(rownames(all.table)),sum)
      cont.table<-do.call(cbind,lapply(sum.table,function(x) cbind(x[,2])))
      rm(all.table,sum.table)
    }
    if(sum(as.vector(cont.table)==0)>=1){
      E<-matrix(0.5,nrow=nrow(cont.table),ncol=ncol(cont.table))
      cont.table<-(E+cont.table)
    }
    if (nrow(cont.table)==2){
      result.oddsratio<-((2*cont.table[1,1]+cont.table[2,1])*cont.table[2,2])/
        ((2*cont.table[1,2]+cont.table[2,2])*cont.table[2,1])
    }
    if (nrow(cont.table)==3){
      result.oddsratio<-((2*cont.table[1,1]+cont.table[2,1])*(2*cont.table[3,2]+cont.table[2,2]))/
        ((2*cont.table[1,2]+cont.table[2,2])*(2*cont.table[3,1]+cont.table[2,1]))
    }
    if (nrow(cont.table)==1){
      result.oddsratio<-NA
    }
    rm(cont.table)
  }
  return(result.oddsratio)
}
