#' OR plot for graphical visualization of fuzzy ZFA optimized region
#'
#' @description This function visualizes the results of optimized region by OR plot in real data analysis. It returns standard ggplot object, upon which the user can add other geoms.
#'
#' @param singleVariantResult results of single variant test, a data frame with three variables: pos, pvalue and OR. The results could be directly obtained from \code{"test.variants"} function.
#' @param zfaResult results of fuzzy ZFA optimization, the output of \code{"fuzzy.zfa"} function.
#' @param geneName Gene name or symbol of the region, if any. Default = NULL.
#' @param bpInfo a vector of bp information for each variant, if any. Default = NULL.
#' @param chr a number or character indicating which chromosome the region belongs to, if any. Default = NULL.
#' @param ... graphical parameters.
#'
#' @details The OR plot is designed to gain more biological insights by graphical visualization for the optimized region after applying fuzzy ZFA.
#'
#' The x axis represents the array and corresponding chromosome position, and the y axis denotes the log transform of odds ratio. Red lines indicate variants of increased risk of disease, while green lines suggest variants with protective effect.
#'
#' The pink bubbles define the p-value of each rare variant, a smaller p-value gives a larger scale. The optimized region is highlighted with labels of both regional odds ratio and p-value.
#'
#' @examples
#' data(fuzzy.zfa.example)
#' attach(fuzzy.zfa.example)
#'
#' # Generate single variant results by wtest
#' result1<-test.variants(Z,y,collapse=FALSE,test = "wtest")
#'
#' # Generate zfa optimization results by wtest
#' result2<-fuzzy.zfa(Z,y,fast.path = TRUE,filter.pval=0.01,test = "wtest")
#'
#' # Visualize the optimization results by OR plot
#' OR.plot<-zfa.plot(singleVariantResult=result1,zfaResult=result2)
#' plot(OR.plot)
#'
#' @export
#' @seealso \code{\link{fuzzy.zfa}}, \code{\link{test.variants}}, \code{\link{odds.ratio}}
#' @import ggplot2
#' @importFrom stats aggregate

# @importFrom ggplot2 geom_point geom_segment scale_size scale_color_manual scale_x_continuous scale_y_continuous labs theme annotate
zfa.plot<-function(singleVariantResult,zfaResult,geneName=NULL,bpInfo=NULL,chr=NULL, ...) {
  # this part generates and transforms the data into what we need
  data<-singleVariantResult
  zfaResult<-zfaResult[,-3]
  zfaResult<-as.matrix(zfaResult)

  ## group the pvalues into five scales according to their sizes for bubble plot
  data$group<-NA
  data$group[which(data$p.value<=0.000001)]<-0
  data$group[which(data$p.value>0.000001 & data$p.value<=0.00001)]<-1
  data$group[which(data$p.value>0.00001 & data$p.value<=0.0001)]<-2
  data$group[which(data$p.value>0.0001 & data$p.value<=0.001)]<-3
  data$group[which(data$p.value>0.001 & data$p.value<=0.01)]<-4
  data$group[which(data$p.value>0.01 & data$p.value<=0.05)]<-5
  data$group[which(data$p.value>0.05 & data$p.value<=0.1)]<-6
  data$group[which(data$p.value>0.1)]<-7
  labs<-data.frame(x1=c(0:7),x2=c("<1e-6","<1e-5","<1e-4","<0.001","0.001-0.01","0.01-0.05","0.05-0.1",">0.1"),stringsAsFactors=FALSE)
  breaks<-sort(unique(data$group))

  ## generate the values and index of lnOR
  data$lnOR<-log(data$odds.ratio)
  data$pos<-c(1:nrow(data))
  data$index<-NA
  data$index[which(data$lnOR>=0)]<-"deleterious"
  data$index[which(data$lnOR<0)]<-"protective"

  ## define the limits of axis
  xmax <- ceiling(max(data$pos))
  xmin <- floor(min(data$pos))
  ymin <- floor(min(data$lnOR))-0.5
  ymax <- ceiling(max(data$lnOR))+0.5

  ## define the ticks and labels on x axis
  xbreak<-floor(seq(xmin,xmax,by=(xmax-xmin)/10))
  if (!is.null(bpInfo)) {
    bp<-round(bpInfo/10^6,digits=2)[xbreak]
    label<-bp
  } else {
    label<-xbreak
  }
  ## generate data for plotting rectangles
  rect3<-data.frame(x1=zfaResult[1],x2=zfaResult[2],
                    y1=-Inf,
                    y2=Inf)
  rect<- data.frame(x=(zfaResult[2]-zfaResult[1])/2+zfaResult[1], y=ifelse(zfaResult[3]>1,ymax,ymin))

  # this part plots the final graph
  ##plot for lnOR: longdash, steelblue
  plot<-ggplot() + geom_segment(data=data,aes(x=pos,xend=pos,y=0,yend=lnOR,colour = index),linetype="solid",size=0.7)+
    scale_color_manual(name="Effect direction",values = c("red2","green3"))

  ## bubble plot for pvalue: radius, color,shape,size and label
  plot<-plot+geom_point(data=data,aes(x=pos,y=lnOR,size=group),colour="white",shape=21,fill="pink",alpha=0.7)
  plot<-plot+scale_size(trans="reverse",range=c(0,7), breaks=breaks,
                        labels=labs$x2[match(breaks,labs$x1)],
                        name="P-value",guide = guide_legend())

  ## set axis format: limits, secondary axis, ticks and labels
  plot<-plot+scale_y_continuous(limits = c(ymin,ymax), breaks=seq(ymin,ymax,1),expand = c(0,0.5),
                                sec.axis = dup_axis(name=NULL,labels = NULL))
  if (is.null(bpInfo)) {
    plot<-plot+scale_x_continuous(limits = c(xmin,xmax),breaks=xbreak,
                                  sec.axis = sec_axis(~.+0,breaks =xbreak,labels=NULL),
                                  expand = c(0,1.5),labels = label)
    plot<-plot+labs(x=paste(geneName,"array position"),y="ln(OR)")
  } else {
    plot<-plot+scale_x_continuous(limits = c(xmin,xmax),breaks=xbreak,
                                  sec.axis = sec_axis(~.+0,breaks =xbreak,name="Array position"),
                                  expand = c(0,1.5),labels = label)
    plot<-plot+labs(x=paste(geneName,"position on chromosome",chr,"(Mb)","\n"),y="ln(OR)")
  }

  ## set background format: axisline size, margin, axis text and others
  plot<-plot + theme(axis.line=element_line(size=0.05,color = "gray50"))+
    theme(plot.margin=unit(c(10,30,10,20),"pt"))
  plot<-plot + theme(panel.background = element_blank(),panel.grid = element_blank())+
    theme(axis.text.x = element_text(angle=60,hjust = 1,color = "black"),
          axis.text.x.top = element_text(angle=0,hjust = 0.5,color = "black"))

  ## set legend format: direction, position, size, and color
  plot<-plot + theme(legend.direction = "horizontal",legend.box = "horizontal",
                     legend.position = "top",legend.box.just = "left",
                     legend.key = element_rect(fill = "white"))

  ## rectangle within optimal region
  plot<-plot+geom_rect(data = rect3,mapping=aes(xmin=x1, xmax=x2, ymin=y1, ymax=y2),fill="lightsteelblue",alpha=0.3)

  ## triangle and text label describing the optimal region: pvalue and odds ratio
  plot<-plot + annotate("text", x=rect[,1],y=rect[,2],
             label=paste("ZFA P-value =",format(zfaResult[4],scientific=TRUE,digit=3),
                         "\n","Regional OR =",zfaResult[3]),
             fontface = "bold",size=3.5)

  ## add horizonal zero line
  plot<-plot+geom_hline(yintercept = 0,linetype="dotted")

  plot<-plot+theme(legend.title=element_text(size=10,family = "sans"),
                   legend.text=element_text(size=10,family = "sans"),
                   text=element_text(size = 12,family = "sans"))
  return(plot)
}

