#' Genomic inflation factor for population stratification control
#'
#' @description This function computes the genomic inflation factor to measure and control the potential inflation of p-values due to population stratification.
#'
#' @param pvalue a vector of ZFA p-values in genome-wide association studies
#'
#' @details To control the potential genomic inflation due to population stratification, we follow Babron M C et. al (2012) and Liu Q et.al (2013) to measure and correct the inflation of p-values. In the absence of stratification, the observed p-values are uniformly distributed, and the -2*log(p-values) follows a 2-df chi-squared distribution. Similar to the Genomic Control lambda, a genomic inflation factor lambda is employed, measuring on the log-transformed ZFA p-values, where \code{lambda = -2median(log(p-values))/1.386294)}. After computing lambda, the p-values could be adjusted by formula \code{exp(log(p-values)/lambda)}.
#'
#'  Note that the ZFA p-value has already been adjusted by Bonferroni correction. In this case, we are likely to obtain a bimodal histogram of p-values with two peaks at 0 and 1, which is due to the multiple correction. Such p-values at 1 are apparently not uniformly distributed, and are therefore filtered when calculating lambda.
#'
#' @return The function returns a list with two components:
#'
#' \item{lambda}{Genomic inflation factor, which is measured as the rartio of median of -2*log(observed p-value) over the expected median of a 2df chi-squared distribution.}
#'
#' \item{pval.corrected}{Corrected p-values adjusting by lambda.}
#'
#' @references Liu Q, Nicolae D L, Chen L S. Marbled Inflation From Population Structure in Gene Based Association Studies With Rare Variants[J]. Genetic epidemiology, 2013, 37(3): 286-292.
#'
#' Babron M C, De Tayrac M, Rutledge D N, et al. Rare and low frequency variant stratification in the UK population: description and impact on association tests[J]. PloS one, 2012, 7(10): e46519.
#'
#' @examples
#'
#' # run genomic control
#' set.seed(1121)
#' pvalue<-rnorm(10000, mean = 0.4, sd=0.1)
#' out<-genomic.control(pvalue)
#'
#' # lambda
#' lambda<-out$lambda
#'
#' # corrected p-values adjusting by lambda
#' pvalue.corrected<-out$pval.corrected
#'
#' @export
#' @importFrom stats median qchisq


genomic.control<-function(pvalue) {
  if (any(is.na(pvalue)))
    stop ("NA occurs in p-value")
  f<-which(pvalue!=1)
  p0<-pvalue[f]
  p1<--2*log(p0)
  lambda<-median(p1)/qchisq(.5, df=2)
  p2<-exp(log(p0)/lambda)
  pvalue[f]<-p2
  result<-list(lambda=lambda,pval.corrected=pvalue)
  return(result)
}




